#!/bin/sh

# Глобальные переменные

IPTABLES="/sbin/iptables"

EXT_INTERFACE="eth0"
INT_INTERFACE="eth1"
DMZ_INTERFACE="eth2"
PPP_INTERFACE="ppp0"
LOC_INTERFACE="lo"

EXT_ADDRESS="213.180.204.11"
INT_ADDRESS="192.168.1.1"
DMZ_ADDRESS="192.168.2.1"
PPP_ADDRESS="192.168.3.1"
LOC_ADDRESS="127.0.0.1"

# Обнуляем таблицы
$IPTABLES -F
$IPTABLES -X
$IPTABLES -t nat -F
$IPTABLES -t nat -X

# Устанавливаем правила по умолчанию для тех пакетов,
# про которые мы случайно забудем при написании правил ;)
$IPTABLES -P INPUT DROP
$IPTABLES -P OUTPUT ACCEPT
$IPTABLES -P FORWARD DROP

# Создаем конечные цепочки для регистрации пакетов
$IPTABLES -N allowed
$IPTABLES -A allowed -j ULOG --ulog-nlgroup 1 --ulog-qthreshold 50 --ulog-prefix allow
$IPTABLES -A allowed -j ACCEPT

$IPTABLES -N rejected
$IPTABLES -A rejected -m limit --limit 5/minute -j LOG --log-prefix "REJECTED: "
$IPTABLES -A rejected -j ULOG --ulog-nlgroup 1 --ulog-qthreshold 50 --ulog-prefix drop
$IPTABLES -A rejected -p tcp -j REJECT --reject-with tcp-reset
$IPTABLES -A rejected -j REJECT --reject-with icmp-port-unreachable

# Правила #############################################################################

# Разрешить все исходящие пакеты
$IPTABLES -A OUTPUT -j allowed

# Разрешить все входящие пакеты из внутренних сетей
$IPTABLES -A INPUT -i $INT_INTERFACE -j allowed
$IPTABLES -A INPUT -i $PPP_INTERFACE -j allowed
$IPTABLES -A INPUT -i $LOC_INTERFACE -j allowed

# Запретить входящие пакеты от нехороших людей
$IPTABLES -A INPUT -s 195.96.72.46 -j rejected

# Разрешить все входящие и транзитные tcp-пакеты, 
# не пытающиеся установить новое соединение
$IPTABLES -A INPUT -p tcp ! --syn -m state --state new -j rejected
$IPTABLES -A INPUT -p tcp -m state --state established,related -j allowed
$IPTABLES -A FORWARD -p tcp ! --syn -m state --state new -j rejected
$IPTABLES -A FORWARD -p tcp -m state --state established,related -j allowed

# Разрешить все входящие UDP-пакеты для DNS
$IPTABLES -A INPUT -p udp --sport 53 -j allowed
$IPTABLES -A INPUT -p udp --dport 53 -j allowed
$IPTABLES -A INPUT -p udp --dport 25001 -j allowed

# Разрешить все входящие UDP-пакеты для Time
$IPTABLES -A INPUT -p udp --sport 123 -j allowed
$IPTABLES -A INPUT -p udp --dport 123 -j allowed

# Разрешить входящие ICMP-пакеты 
$IPTABLES -A INPUT -p icmp -j allowed

# Разрешить входящие TCP-пакеты для SSH
$IPTABLES -A INPUT -p tcp --syn --dport 22 -j allowed

# Разрешить входящие TCP-пакеты для SMTP
$IPTABLES -A INPUT -p tcp --syn --dport 25 -j allowed

# Проброс портов для машины из DMZ
$IPTABLES -A INPUT -i $EXT_INTERFACE -p TCP --syn --dport 2022 -j allowed
$IPTABLES -t nat -A PREROUTING -p tcp -d $EXT_ADDRESS --dport 2022 -j DNAT --to-destination 192.168.2.2:22
$IPTABLES -A FORWARD -o $DMZ_INTERFACE -i $EXT_INTERFACE -p TCP --syn -d 192.168.2.2 --dport 22 -j allowed

# Хосты внутренних сетей, которым разрешен выход в интернет
NAT_EXT="192.168.1.2 192.168.1.3 192.168.3.2"

for i in  $NAT_EXT
do    
    $IPTABLES -t nat -A POSTROUTING -s $i -o $EXT_INTERFACE -j SNAT --to-source $EXT_ADDRESS    
    $IPTABLES -A FORWARD -s $i -o $EXT_INTERFACE -j allowed
    $IPTABLES -A FORWARD -p tcp ! --syn -d $i -i $EXT_INTERFACE -j allowed
    $IPTABLES -A FORWARD -p udp --dport 53 -d $i -i $EXT_INTERFACE -j allowed
    $IPTABLES -A FORWARD -p udp --dport 123 -d $i -i $EXT_INTERFACE -j allowed
    $IPTABLES -A FORWARD -p icmp -d $i -i $EXT_INTERFACE -j allowed
done

# Запретить все остальные входящие и транзитные пакеты
$IPTABLES -A INPUT -j rejected
$IPTABLES -A FORWARD -j rejected
