create type summary_ip_direction as(sumdate date, intf varchar(15), address inet,
	allow bigint, drop bigint);

create or replace function summary_ip_in(varchar(15), boolean) returns setof summary_ip_direction as $$
    select cast(date_trunc('day', packetdate) as date), dintf, dst,
        cast(sum(case when prefix='allow' then bytes else 0 end) as bigint),
        cast(sum(case when prefix='drop' then bytes else 0 end) as bigint)
        from transfer_ip
        where sintf=$1 and dintf<>$1 and processed=$2
        group by 1, 2, 3;
$$ language sql;

create or replace function summary_ip_out(varchar(15), boolean) returns setof summary_ip_direction as $$
    select cast(date_trunc('day', packetdate) as date), sintf, src,
        cast(sum(case when prefix='allow' then bytes else 0 end) as bigint),
        cast(sum(case when prefix='drop' then bytes else 0 end) as bigint)
        from transfer_ip
        where dintf=$1 and sintf<>$1 and processed=$2
        group by 1, 2, 3;
$$ language sql;

create type summary_ip_nodirection as(sumdate date, intf varchar(15), address inet,
	in_allow bigint, in_drop bigint, out_allow bigint, out_drop bigint);

create or replace function summary_ip_routed(varchar(15), boolean) returns setof summary_ip_nodirection as $$
    select 
        case when summary_ip_in.sumdate is null then summary_ip_out.sumdate else summary_ip_in.sumdate end, 
        case when summary_ip_in.intf is null then summary_ip_out.intf else summary_ip_in.intf end, 
        case when summary_ip_in.address is null then summary_ip_out.address else summary_ip_in.address end, 
        case when summary_ip_in.allow is null then 0 else summary_ip_in.allow end,
        case when summary_ip_in.drop is null then 0 else summary_ip_in.drop end,
        case when summary_ip_out.allow is null then 0 else summary_ip_out.allow end,
        case when summary_ip_out.drop is null then 0 else summary_ip_out.drop end
        from summary_ip_in($1, $2) full join summary_ip_out($1, $2)
        on summary_ip_out.sumdate = summary_ip_in.sumdate
        and summary_ip_out.intf = summary_ip_in.intf
        and summary_ip_out.address = summary_ip_in.address;
$$ language sql;

create or replace function summary_ip_local(varchar(15), boolean) returns setof summary_ip_nodirection as $$
    select cast(date_trunc('day', packetdate) as date), 'same', dst,
        cast(sum(case when prefix='allow' then bytes else 0 end) as bigint),
        cast(sum(case when prefix='drop' then bytes else 0 end) as bigint),
        cast(0 as bigint), 
        cast(0 as bigint)
        from transfer_ip
        where dintf=$1 and sintf=$1 and processed=$2
        group by 1, 2, 3;
$$ language sql;

create or replace function summary_ip_calculate(varchar(15)) returns void as $$
declare
    r summary_ip_nodirection;
begin
    for r in 
        select * from summary_ip_local($1, false) union 
        select * from summary_ip_routed($1, false)
    loop
        perform * from summary_ip 
            where sumdate=r.sumdate and intf=r.intf and address=r.address;
        if found then
            --raise notice 'summary ip record [%,%,%] is found, so it will be updated', r.sumdate, r.intf, r.address;
            update summary_ip 
                set in_allow=in_allow+r.in_allow, in_drop=in_drop+r.in_drop, out_allow=out_allow+r.out_allow, out_drop=out_drop+r.out_drop
                where sumdate=r.sumdate and intf=r.intf and address=r.address;
        else
            --raise notice 'summary ip record [%,%,%] is not found, so it will be inserted', r.sumdate, r.intf, r.address;
            insert into summary_ip values (r.sumdate, r.intf, r.address,
                r.in_allow, r.in_drop, r.out_allow, r.out_drop);
        end if;
    end loop;
    update transfer_ip set processed = true where processed = false;
end
$$ language plpgsql security definer;
